"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CurrentVitalsStatus = void 0;

var _preact = require("preact");

var _classnames = _interopRequireDefault(require("classnames"));

var _milliseconds = _interopRequireDefault(require("milliseconds"));

var _icon = require("./icon");

var _date = require("./utils/date");

var _bpm = require("./utils/bpm");

var _vitalTypes = require("./utils/vital-types");

var _getBpString = _interopRequireDefault(require("./utils/get-bp-string"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const maxAge = _milliseconds.default.seconds(60);

const delayedAge = _milliseconds.default.seconds(20);

const veryDelayedAge = _milliseconds.default.seconds(30);

const defaultMaxVitalAge = _milliseconds.default.seconds(60);

const defaultMaxBpAge = _milliseconds.default.minutes(6);

const withoutExpired = ({
  vitals,
  now,
  maxAge = defaultMaxVitalAge,
  maxBpAge = defaultMaxBpAge
}) => {
  const newVitals = _objectSpread({}, vitals);

  for (const [id, vital] of Object.entries(vitals)) {
    const threshold = 'bp' === id ? maxBpAge : maxAge;

    if (vital.time && now - vital.time > threshold) {
      newVitals[id] = _objectSpread(_objectSpread({}, vital), {}, {
        value: null
      });
    }
  }

  return newVitals;
};

const VitalEntry = ({
  id,
  name,
  value,
  unit = null,
  loading,
  error
}) => {
  let content;

  if (value == null && !error) {
    error = true;
  } // Cast BP object to string


  if (id === 'bp' && value && value.sbp != null) {
    value = (0, _getBpString.default)(value);
  }

  if (loading) {
    content = (0, _preact.h)(_icon.Icon, {
      className: "spin-animation",
      size: "16",
      icon: "autorenew"
    });
  } else if (error) {
    content = (0, _preact.h)("div", {
      "data-e2e": "error",
      className: "flex items-center"
    }, (0, _preact.h)(_icon.Icon, {
      size: "16",
      icon: "error"
    }), error !== true ? (0, _preact.h)("span", {
      className: "ml1"
    }, error) : null);
  } else {
    content = (0, _preact.h)("strong", {
      "data-e2e": "value"
    }, value);
  }

  return (0, _preact.h)("div", {
    "data-e2e": "vital|".concat(name),
    className: (0, _classnames.default)('f6 fw4 flex items-center mr1', {
      'bg-red white br1 pa1': error
    })
  }, (0, _preact.h)("span", {
    className: "mr1"
  }, name, ":"), content, unit ? " ".concat(unit) : '');
};
/**
 * @typedef {{
 *   currentVitals: import('./current-vitals-status.types').CurrentVitalsDoc
 *   isLocal: boolean
 *   extraContent?: h.JSX.Element
 *   className?: string
 *   hideLeadingIcon?: boolean
 *   appTime?: number
 *   maxVitalAge?: number
 *   maxBpAge?: number
 * }} Props
 */

/**
 * @typedef {{
 *   now?: number
 * }} State
 */

/** @extends {Component<Props, State>} */


class CurrentVitalsStatus extends _preact.Component {
  constructor(props) {
    super(props);
    this.state = {
      now: Date.now()
    };
    this.refreshTimer = null;
  }

  startTimer() {
    if (!this.refreshTimer && !this.props.appTime) {
      this.refreshTimer = setInterval(() => this.setState({
        now: Date.now()
      }), 1000);
    }
  }

  componentWillUnmount() {
    if (this.refreshTimer) {
      clearInterval(this.refreshTimer);
    }
  }

  render() {
    const {
      currentVitals,
      isLocal,
      extraContent,
      className,
      hideLeadingIcon = false
    } = this.props;
    const now = this.props.appTime || this.state.now;

    if (!currentVitals) {
      clearTimeout(this.refreshTimer);
      this.refreshTimer = null;
      return null;
    }

    const {
      lastUpdated,
      simulated,
      monitorLabel,
      autoVitalsStatus
    } = currentVitals;
    const vitals = withoutExpired({
      vitals: currentVitals.vitals || {},
      now,
      maxAge: this.props.maxVitalAge,
      maxBpAge: this.props.maxBpAge
    });
    const age = now - lastUpdated;

    if (age > maxAge && !isLocal) {
      return null;
    }

    this.startTimer();
    let isVeryDelayed = !isLocal ? age > veryDelayedAge : false;
    let isSomewhatDelayed = !isLocal && !isVeryDelayed ? age > delayedAge : false;
    const isConnected = autoVitalsStatus === 'connected';
    const isWaiting = autoVitalsStatus === 'waiting';
    const hasValues = Object.values(vitals).some(vital => vital.value != null); // If we were given a list of selectedVitals show them all, otherwise show all vitals we have values for

    const selectedVitalIds = (currentVitals.selectedVitals || Object.keys(vitals)).sort((a, b) => _vitalTypes.vitalIdToSortOrder[a] - _vitalTypes.vitalIdToSortOrder[b]);
    return (0, _preact.h)("div", {
      "data-e2e": "currentVitalsStatus",
      className: (0, _classnames.default)('flex items-center flex-wrap justify-between', className)
    }, (0, _preact.h)("div", {
      className: "flex items-center"
    }, !hideLeadingIcon && (0, _preact.h)("div", null, isConnected && (0, _preact.h)(_icon.Icon, {
      className: "nl1 ".concat(isLocal ? 'dark-blue' : 'light-blue'),
      size: "48",
      icon: "live_dot",
      speed: (0, _bpm.bpmToPlaybackSpeed)(vitals.hr && vitals.hr.value),
      "data-e2e": "currentVitalsStatusConnected"
    }), isWaiting && (0, _preact.h)(_icon.Icon, {
      "data-e2e": "currentVitalsStatusConnecting",
      size: "24",
      icon: "autorenew",
      className: "spin-animation"
    })), (0, _preact.h)("div", {
      className: "pa1"
    }, (0, _preact.h)("div", {
      className: "flex items-center flex-wrap"
    }, (0, _preact.h)("div", null, (0, _preact.h)("div", {
      className: "flex items-center fw6 nowrap"
    }, (0, _preact.h)("span", {
      className: "mr1"
    }, monitorLabel, " Data"), simulated && (0, _preact.h)("span", {
      className: "mr1"
    }, "(simulated)"), !isLocal && (0, _preact.h)("span", {
      className: "mr1"
    }, "(via other computer)"))), !isWaiting && (0, _preact.h)("div", {
      className: (0, _classnames.default)('nowrap mr1', {
        orange: isSomewhatDelayed,
        red: isVeryDelayed
      })
    }, "Received: ", lastUpdated ? "".concat((0, _date.msToTimeAgo)(age), " ago") : '(none)', ","), !isWaiting && (0, _preact.h)("div", {
      className: "nowrap mr1"
    }, "Last BP received:", ' ', vitals.bp ? "".concat((0, _date.msToTimeAgo)(now - vitals.bp.time), " ago") : '(none)')), !isWaiting && (0, _preact.h)("div", null, (0, _preact.h)("div", {
      className: "flex flex-wrap mt1"
    }, selectedVitalIds.map(id => {
      const vital = vitals[id];
      const vitalType = _vitalTypes.vitalTypesById[id];
      return (0, _preact.h)(VitalEntry, {
        key: id,
        id: id,
        name: vitalType && vitalType.chartName || id,
        value: vital && vital.value,
        error: vital && vital.error,
        loading: vital && vital && vital.status === 'measuring'
      });
    }))), !hasValues && isConnected && (0, _preact.h)("div", {
      className: "inline-flex items-center mt1"
    }, (0, _preact.h)(_icon.Icon, {
      icon: "warning",
      size: "16",
      className: "mr1"
    }), (0, _preact.h)("span", null, "The monitor is connected but sending blank vitals values.")))), extraContent, !isLocal && (isVeryDelayed || isSomewhatDelayed) && (0, _preact.h)("div", {
      className: (0, _classnames.default)('br2 pa2 flex items-center', {
        'bg-red white': isVeryDelayed,
        'bg-orange': isSomewhatDelayed
      })
    }, (0, _preact.h)(_icon.Icon, {
      icon: "warning",
      className: "mr2"
    }), "Data is delayed"));
  }

}

exports.CurrentVitalsStatus = CurrentVitalsStatus;